@@ -36,7 +36,6 @@
 #include "nautilus-window.h"
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo/bonobo-main.h>
-#include <bonobo/bonobo-ui-main.h>
 #include <dlfcn.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-glib-extensions.h>
@@ -55,6 +54,7 @@
 #include <libxml/parser.h>
 #include <popt.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 /* Keeps track of everyone who wants the main event loop kept active */
@@ -136,34 +136,112 @@ register_icons (void)
 	GtkIconSource *source;
 	GtkIconSet *set;
 	GtkIconFactory *factory;
+	const char *icons_to_register[] = {"gnome-fs-client", "gnome-fs-trash-empty", "gnome-dev-cdrom", "stock_new-template"};
+	struct { int pixel; int gtk; } sizes[] = {
+		{16, GTK_ICON_SIZE_MENU},
+		{18, GTK_ICON_SIZE_SMALL_TOOLBAR},
+		{20, GTK_ICON_SIZE_BUTTON},
+		{24, GTK_ICON_SIZE_LARGE_TOOLBAR},
+		{32, GTK_ICON_SIZE_DND},
+		{48, GTK_ICON_SIZE_DIALOG},
+		{48, 0},
+	};
+	int i, j;
 
 	icon_theme = nautilus_icon_factory_get_icon_theme ();
-	info = gtk_icon_theme_lookup_icon (icon_theme, "gnome-fs-client", 48,
-					   0);
-	if (info != NULL) {
-		icon = gtk_icon_info_get_filename (info);
-		factory = gtk_icon_factory_new ();
-		gtk_icon_factory_add_default (factory);
-		
-		source = gtk_icon_source_new ();
-		gtk_icon_source_set_filename (source, icon);
-		
+
+	factory = gtk_icon_factory_new ();
+	gtk_icon_factory_add_default (factory);
+	for (i = 0; i < G_N_ELEMENTS(icons_to_register); i++) {
+
 		set = gtk_icon_set_new ();
-		gtk_icon_set_add_source (set, source);
+		source = gtk_icon_source_new ();
+
+		for (j = 0; j < G_N_ELEMENTS(sizes); j++) {
+			info = gtk_icon_theme_lookup_icon (icon_theme, icons_to_register[i], sizes[j].pixel, 0);
+			
+			if (info != NULL &&
+			    (sizes[j].gtk == 0 ||
+			     gtk_icon_info_get_base_size (info) == sizes[j].pixel)) {
+				icon = gtk_icon_info_get_filename (info);
+				gtk_icon_source_set_filename (source, icon);
+				if (sizes[j].gtk == 0) {
+					gtk_icon_source_set_size (source, 0);
+					gtk_icon_source_set_size_wildcarded (source, TRUE);
+				} else {
+					gtk_icon_source_set_size (source, sizes[j].gtk);
+					gtk_icon_source_set_size_wildcarded (source, FALSE);
+				}
+				gtk_icon_set_add_source (set, source);
+			}
+			if (info != NULL) {
+				gtk_icon_info_free (info);
+			}
+		}
 
-		gtk_icon_factory_add (factory, "gnome-fs-client", set);
-		gtk_icon_set_unref (set);
-		
 		gtk_icon_source_free (source);
+		gtk_icon_factory_add (factory, icons_to_register[i], set);
+		gtk_icon_set_unref (set);
 
-		gtk_icon_info_free (info);
-		g_object_unref (factory);
 	}
-	
+	g_object_unref (factory);	
 	g_object_unref (icon_theme);
 	
 }
 
+/* Copied from libnautilus/nautilus-program-choosing.c; In this case,
+ * though, it's really needed because we have no real alternative when
+ * no DESKTOP_STARTUP_ID (with its accompanying timestamp) is
+ * provided...
+ */
+static Time
+slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
+{
+	Window xwindow;
+	XEvent event;
+	
+	{
+		XSetWindowAttributes attrs;
+		Atom atom_name;
+		Atom atom_type;
+		char* name;
+		
+		attrs.override_redirect = True;
+		attrs.event_mask = PropertyChangeMask | StructureNotifyMask;
+		
+		xwindow =
+			XCreateWindow (xdisplay,
+				       RootWindow (xdisplay, 0),
+				       -100, -100, 1, 1,
+				       0,
+				       CopyFromParent,
+				       CopyFromParent,
+				       CopyFromParent,
+				       CWOverrideRedirect | CWEventMask,
+				       &attrs);
+		
+		atom_name = XInternAtom (xdisplay, "WM_NAME", TRUE);
+		g_assert (atom_name != None);
+		atom_type = XInternAtom (xdisplay, "STRING", TRUE);
+		g_assert (atom_type != None);
+		
+		name = "Fake Window";
+		XChangeProperty (xdisplay, 
+				 xwindow, atom_name,
+				 atom_type,
+				 8, PropModeReplace, name, strlen (name));
+	}
+	
+	XWindowEvent (xdisplay,
+		      xwindow,
+		      PropertyChangeMask,
+		      &event);
+	
+	XDestroyWindow(xdisplay, xwindow);
+	
+	return event.xproperty.time;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -172,6 +250,8 @@ main (int argc, char *argv[])
 	gboolean no_default_window;
 	gboolean browser_window;
 	gboolean no_desktop;
+	const char *startup_id;
+	char *startup_id_copy;
 	char *geometry;
 	gboolean perform_self_check;
 	poptContext popt_context;
@@ -222,6 +302,17 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	startup_id = g_getenv ("DESKTOP_STARTUP_ID");
+	startup_id_copy = NULL;
+	if (startup_id != NULL && *startup_id != '\0') {
+		/* Clear the DESKTOP_STARTUP_ID, but make sure to copy it first */
+		startup_id_copy = g_strdup (startup_id);
+		putenv ("DESKTOP_STARTUP_ID=");
+	}
+
+	/* we'll do it ourselves due to complicated factory setup */
+	gtk_window_set_auto_startup_notification (FALSE);
+
 	/* Get parameters. */
 	geometry = NULL;
 	kill_shell = FALSE;
@@ -240,7 +331,19 @@ main (int argc, char *argv[])
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Nautilus"),
 				      NULL);
 
+	/* Do this here so that gdk_display is initialized */
+	if (startup_id_copy == NULL) {
+		/* Create a fake one containing a timestamp that we can use */
+		Time timestamp;
+		timestamp = slowly_and_stupidly_obtain_timestamp (gdk_display);
+		startup_id_copy = g_strdup_printf ("_TIME%lu",
+						   timestamp);
+	}
+
 	register_icons ();
+
+        /* Set default icon for all nautilus windows */
+	gtk_window_set_default_icon_name ("gnome-fs-directory");
 	
 	/* Need to set this to the canonical DISPLAY value, since
 	   thats where we're registering per-display components */
@@ -283,10 +386,6 @@ main (int argc, char *argv[])
 	/* Initialize the services that we use. */
 	LIBXML_TEST_VERSION
 
-	if (g_getenv ("NAUTILUS_ENABLE_TEST_COMPONENTS") != NULL) {
-		bonobo_activation_set_test_components_enabled (TRUE);
-	}
-
 	/* Initialize preferences. This is needed so that proper 
 	 * defaults are available before any preference peeking 
 	 * happens.
@@ -326,6 +425,7 @@ main (int argc, char *argv[])
 			 kill_shell, restart_shell, no_default_window, no_desktop,
 			 !(kill_shell || restart_shell),
 			 browser_window,
+			 startup_id_copy,
 			 geometry,
 			 args);
 		if (is_event_loop_needed ()) {
@@ -345,7 +445,6 @@ main (int argc, char *argv[])
 	}
 
 	eel_debug_shut_down ();
-	bonobo_ui_debug_shutdown ();
 
 	/* If told to restart, exec() myself again. This is used when
 	 * the program is told to restart with CORBA, for example when
